﻿using SqlSugar;
using System;
using System.Collections.Generic;
using System.Text;

namespace wms.sqlsugar.model.cp
{
    /// <summary>
    ///  甲方管理系统推送反馈
    /// </summary>
    [Tenant("cp")]
    [SugarTable(nameof(cpBoxItemsHty) + "_{year}{month}{day}", "箱号明细表")]
    [SplitTable(SplitType.Year)]//按年分表 （自带分表支持 年、季、月、周、日）
    public class cpBoxItemsHty:BaseModel
    {
        /// <summary> 
        /// 箱号组Id（关联组表）
        /// </summary> 
        [SugarColumn(ColumnDataType = "bigint", IsNullable = false)]
        public long CongBoxId { get; set; }
        /// <summary> 
        /// 箱号
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string HuNr { get; set; }

        /// <summary> 
        /// 工字轮号
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string HWBarCode { get; set; }

        /// <summary> 
        /// 库存状态（锁定/正常）
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string ExecStateCode { get; set; }

        /// <summary> 
        /// 生产订单号
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string OrderNo { get; set; }

        /// <summary> 
        /// 生产订单行号
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string OrderRowNo { get; set; }

        /// <summary> 
        /// 组盘状态
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string InvStateCode { get; set; }

        /// <summary> 
        /// 入库单据号
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string InDocsNo { get; set; }

        /// <summary> 
        /// 入库单行号
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string InDocsRowNo { get; set; }

        /// <summary> 
        /// 供应商编码
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string SuppCode { get; set; }

        /// <summary> 
        /// 供应商名称
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string SuppName { get; set; }

        /// <summary> 
        /// 客户编码
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string CustCode { get; set; }

        /// <summary> 
        /// 客户名称
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string CustName { get; set; }

        /// <summary> 
        /// MatId
        /// </summary> 
        [SugarColumn(ColumnDataType = "bigint", IsNullable = false)]
        public long MatId { get; set; }

        /// <summary> 
        /// MatCode
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string MaterialCode { get; set; }

        /// <summary> 
        /// MatName
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 100, IsNullable = true)]
        public string MaterialDesc { get; set; }

        /// <summary> 
        /// 总重量
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string HuTotalWt { get; set; }

        /// <summary> 
        /// 净重
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string HuNetWt { get; set; }

        /// <summary> 
        /// 皮重
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string HuTareWt { get; set; }

        /// <summary> 
        /// 焊点数量
        /// </summary> 
        [SugarColumn(ColumnDataType = "decimal", Length = 18, DecimalDigits = 6, IsNullable = false)]
        public decimal BadpointCount { get; set; }

        /// <summary> 
        /// 批号
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string Batch { get; set; }

        /// <summary> 
        /// 钢丝质量等级A/B/C/废品
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string GradeCode { get; set; }

        /// <summary> 
        /// 绕向
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string Winding { get; set; }

        /// <summary> 
        /// 是否做扭转检测 0 不做；1 做
        /// </summary> 
        [SugarColumn(ColumnDataType = "decimal", Length = 18, IsNullable = false)]
        public decimal OvcFlag { get; set; }

        /// <summary> 
        /// 起鼓等级
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string PlumpGrade { get; set; }

        /// <summary> 
        /// 扭转检测结果值
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string OvcValue { get; set; }

        /// <summary> 
        /// 装箱时间
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 100, IsNullable = true)]
        public string ProduceDate { get; set; }

        /// <summary> 
        /// 工序订单号
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string OrderProcessNo { get; set; }

        /// <summary> 
        /// 机器组编码
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string MachineGroupcode { get; set; }

        /// <summary> 
        /// 产线
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string ProdLine { get; set; }

        /// <summary> 
        /// 产线代码
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string ProdLineCode { get; set; }

        /// <summary>
        /// 1 放行/ 2 冻结 ERP使用
        /// </summary>
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string State { get; set; }

        /// <summary>
        /// 销售发货单预冻结
        /// </summary>
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string SalesDoc { get; set; }

        /// <summary>
        /// Sku
        /// </summary>
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string Sku { get; set; }

        /// <summary>
        /// 班次
        /// </summary>
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string ShiftNo { get; set; }

        /// <summary>
        /// 班别
        /// </summary>
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string ShiftGroup { get; set; }

        /// <summary>
        /// 材料号
        /// </summary>
        [SugarColumn(ColumnDataType = "nvarchar", Length = 200, IsNullable = true)]
        public string CLBarCode { get; set; }

        /// <summary>
        /// 码垛工位
        /// </summary>
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string EquNo { get; set; }


        /// <summary>
        /// 推送状态（0未上传1上传成功 2上传失败）
        /// </summary>
        [SugarColumn(ColumnDataType = "int", IsNullable = false)]
        public int UpLoadFlag { get; set; }

        /// <summary>
        /// 是否需要上传
        /// </summary>
        [SugarColumn(ColumnDataType = "int", IsNullable = false)]
        public int IsNeedUpLoadFlag { get; set; }

        /// <summary>
        /// 码垛层
        /// </summary>
        [SugarColumn(ColumnDataType = "int", IsNullable = false)]
        public int Layer { get; set; }
        /// <summary>
        ///  是否黑盘
        /// </summary>
        [SugarColumn(ColumnDataType = "bit", IsNullable = true)]
        public bool IsBlack { get; set; }
        /// <summary>
        ///  是否控制盘
        /// </summary>
        [SugarColumn(ColumnDataType = "bit", IsNullable = true)]
        public bool IsControlpanel { get; set; }
        /// <summary>
        /// 单爪码垛完成时间
        /// </summary>
        [SugarColumn(ColumnDataType = "datetime", IsNullable = true)]
        public DateTime PalletizingDate { get; set; }

        /// <summary>
        /// 时间戳
        /// </summary>
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true)]
        public string TimeStamp { get; set; }
    }
}
